//=============================================================================
//  MuseScore
//  Linux Music Score Editor
//  $Id:$
//
//  Erhu (Niko) Notation plugin
//
//  Copyright (C)2011-2014 Hiroshi Tachibana
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License version 2.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//=============================================================================
//
// This is ECMAScript code (ECMA-262 aka "Java Script")
//
//---------------------------------------------------------
//    init
//    this function will be called on startup of mscore
//---------------------------------------------------------
var fingerings = [ "A", "B", "C", "D", "E", "F", "G", "H", "I", "J", "K", "L", 
"M", "N", "O", "P", "Q", "R", "S", "T", "U", "V", "W", "X",
"Y", "Z", "a", "b", "c", "d", "e", "f", "g", "h", "i", "j", "k", "l", 
"m", "n", "o", "p", "q","r","s","t","u","v","x","y","z",
"0","1","2","3","4","5","6","7","8"] 
// Aは0		１番目
// Bは1..	２番目
// Nは1		１４番目    1=Dの場合、MIDIノート番号は、中央のD=62?  50
// Zは1^・	２６番目
// lは1^・・	３８番目
// xは1^・・・	５０番目
// 8は7^・・・ ６１番目


function init(){}
//-------------------------------------------------------------------
//    run
//    this function will be called when activating the
//    plugin menu entry
//
//    global Variables:
//    pluginPath - contains the plugin path; file separator is "/"
//-------------------------------------------------------------------
function run(){
	if (typeof curScore === 'undefined') return;
	var loader = new QUiLoader(null);
	var file   = new QFile(pluginPath + "/Erhu.ui"); // 外部Qtファイルの指定
	file.open(QIODevice.OpenMode(QIODevice.ReadOnly, QIODevice.Text));
	form = loader.load(file, null);
	form.buttonBox.accepted.connect(accept);
	form.exec();
}
//---------------------------------------------------------
//    accept
//    called when user presses "Accept" button
//---------------------------------------------------------
function accept(){
	var ft1 = form.radioButton_1.checked; // 二胡のキー　1=D (1 5)かどうかのスイッチ
	var ft2 = form.radioButton_2.checked; // 1=G (5 2)
	var ft3 = form.radioButton_3.checked; // 1=F
	var ft4 = form.radioButton_4.checked; // 1=C (2 6)
	var ft5 = form.radioButton_5.checked; // 1=Bb
	var ft6 = form.radioButton_6.checked; // 1=C (1 5)
	var ft7 = form.radioButton_7.checked; // 1=G (1 5)
	print(ft1+" "+ft2+" "+ft3+" "+ft4+" "+ft5+" "+ft6+" "+ft7);
	var ftS = form.radioButton_S.checked; // ♯を使う場合
	var ftF = form.radioButton_F.checked; // ♭を使う場合
	print(ftS+" "+ftF);
	var fontSize = form.mSpinBox3.value; // フォントサイズの数値を得る
	var fontYOffset = form.mSpinBox4.value; // フォント位置の数値を得る

// 二胡の調弦は、常にDとAである。キーによってポジションの読み方が変わるタブ譜である。
	if(ft1)      var key= 0; // 1=D (1  5)　キーの指定  五線譜のキーはDでなければならない。
	else if(ft2) var key=-5; // 1=G (5. 2)
	else if(ft3) var key=-3; // 1=F (6. 3)
	else if(ft4) var key= 2; // 1=C (2  6)
	else if(ft5) var key= 3; // 1=Bb(3. 7.)
	else if(ft6) var key= 2; // 1=C (1  5)  // 1=A (
	else if(ft7) var key=-5; // 1=G (1  5)  // 1=Eb(

	if(ft1 || ft2 || ft3 || ft4 || ft5) var lowLimit=62;
	else if(ft6) var lowLimit=60;
	else if(ft7) var lowLimit=55;

	if(ftS) font = new QFont("ErhuS", fontSize); // ♯♭フォントを定義
	else if(ftF) font = new QFont("ErhuF", fontSize); // ♯フォントを定義

	var CrLf = '\r\n'; // 和音を縦に表示するための改行用文字列
	var select=true; // 選択があるorないの判定用の論理型変数の定義
	var currentCursor = new Cursor(curScore);
	var selectionEnd = new Cursor(curScore);
	selectionEnd.goToSelectionEnd(); // 選択範囲の末端に移動
	var startStaff=currentCursor.staff; // 選択範囲の先頭小節
	var endStaff=selectionEnd.staff; // 選択範囲の末端小節

//	currentCursor.staff = 0; // 多段のスコアには自動的に対応せず（各段を選択することにより表示ができる）
//	currentCursor.voice = 0; // voice2〜4には対応せず
	currentCursor.rewind();  // 先頭に戻す

	currentCursor.goToSelectionStart(); // 選択範囲の先頭に移動
	if (currentCursor.eos()) { // 選択がない場合は、currentCursorは末端となる
		select=false; // 選択がない
		startStaff = 0; // 範囲の先頭は、最初の小節
		endStaff = curScore.staves; // 範囲の末尾は、最後の小節
	}
	currentCursor.goToSelectionStart(); // 先頭を現在位置にする
	if(!select)  currentCursor.rewind(); // 選択がないときは、先頭に戻して先頭から.
	while (currentCursor.tick() < selectionEnd.tick()) { // 現在位置が選択範囲内にいる場合
		if (currentCursor.isChord()) { // 現在位置が音符の場合
			var text  = new Text(curScore);
			var chordLength = currentCursor.chord().notes; // 和音の音数を取得
			var len = chordLength-1; // 和音の音数を０から数えた場合
			while (len >= 0){ // 音符の読みを下から並べるための和音の音数のループ
				var pitch = currentCursor.chord().note(len).pitch; // 音符の音程の取得
				var index = pitch-50+1; // 1=Dの時のfingerings配列の"1"の位置。

// 1=Dのとき、最低音Dの"１"のノートNo.は、62。これ以下の音程は表示しない。（通常）
// 1=Gのとき、最低音Gの"５."のノートNo.は、62。これ以下の音程は表示しない。（通常）
// 1=Fのとき、最低音Fの"６."のノートNo.は、62。　　（通常）
				var index = pitch -50 +1 +key;
				if(pitch >= lowLimit && index < fingerings.length-1){  // 音程が表示可能範囲内である場合
					text.text += fingerings[index]+CrLf; // 数字譜の文字設定
				}
//				}else if(ft5) text.text += currentCursor.chord().note(len).name+CrLf; // 英語音名の文字設定（移調しない）
//				else if(ft6) text.text += fingerings[(pitch-shinobueLength+8) % 12]+CrLf; // ドレミ#音名の文字設定（移調する）
//				else if(ft7) text.text += fingerings[(pitch-shinobueLength+8) % 12]+CrLf; // ドレミb音名の文字設定（移調する）

				len = len-1; // 和音の音数のカウントを減らす
			}
			text.defaultFont = font; // フォントの設定
			text.yOffset = fontYOffset; // 縦方向の位置設定
			currentCursor.putStaffText(text); // 画面に音符を表示
		}
		currentCursor.next(); // 次の音符に進める
	}
}

//---------------------------------------------------------
//    menu:  defines were the function will be placed
//           in the MuseScore menu structure
//---------------------------------------------------------

var mscorePlugin = {
      menu: 'Plugins.Erhu Notation', // MuseScoreのプラグインメニューに表示されるタイトル
      init: init,
      run:  run
      };

mscorePlugin;
